///
/// @file       QDrawingArea.hpp
/// @author     Armin Co
///
/// @brief      QDrawingArea is a class which can show a Gdk::Pixbuf.
///

#ifndef QPONG_DRAWING_AREA_HPP
#define QPONG_DRAWING_AREA_HPP

#include <gtkmm.h>
#include <gdkmm.h>


/// @brief  QDrawingArea is derived from Gtk::DrawingArea.
///         An object of this class stores a reference to Gdk::Pixbuf which can be shown and updated.
///
class QDrawingArea : public Gtk::DrawingArea
{
public:
    QDrawingArea();


    virtual ~QDrawingArea(){};


    /// @brief Request a redraw of the drawing area.
    ///
    void redraw();


    /// @brief Set a new pixbuf reference.
    ///
    void setPixbuf(Glib::RefPtr<Gdk::Pixbuf> pixbuf);


protected:
    /// @brief Signal handler which is called when a redraw of the drawing area is requested.
    ///
    bool on_draw(const Cairo::RefPtr<Cairo::Context> &cr) override;


private:
    Glib::RefPtr<Gdk::Pixbuf> m_image; ///< Reference a pixbuf.
    Glib::Dispatcher m_dispatcher; ///< Dispatcher is used to trigger thread safe redraws.
};

#endif