///
/// @file       Common.hpp
/// @author     Armin Co
///

#ifndef QPONG_COMMON_HPP
#define QPONG_COMMON_HPP

namespace QPong
{

const int arrayWidth = 384;
const int arrayHeight = 384;
const int numberOfThreads = 8;
const unsigned arraySize = arrayWidth * arrayHeight;

//static_assert(arraySize % numberOfThreads == 0, "Number of threads and array size doesn't match!");


/// @brief ImageBuffer params
constexpr double ImageBufferMaxColorValue = 0.81;


/// @brief Particle params
constexpr double Particle_hbar = 0.0002;
constexpr double ParticleInitialImpulseX = 0.063;
constexpr double ParticleInitialImpulseY = 0.042;
constexpr double ParticleInitialPosX = 0.0;
constexpr double ParticleInitialPosY = 0.0;
constexpr double ParticleInitialLambdaUnsharpness = 60.0;
constexpr double ParticleIntialValueScaleFactor = 0.5;

constexpr double ParticleBatScale = 0.000015;
constexpr double ParticleBatHeight = 0.31;
constexpr double ParticleBatCutOff = 0.0015;

constexpr double ParticleTimeStepSize = 0.15;

constexpr double ParticleFlattenMomentumCutOff = 0.012;
constexpr double ParticleFlattenMomentumFallOff = 2.0;

constexpr double ParticleMomentumImageScale = 55580.0;
}

///  @brief Calc square of a number.
///
template <typename T>
const T sqr(const T n)
{
    return n * n;
}

#endif