///
/// @file   Config.cpp
/// @author Armin Co
///

#include "Config.hpp"

#include<iostream>
#include<fstream>

using namespace QPong;

const char* toStr(Setting s)
{
    switch (s)
    {
    case Setting::ArrayHeight : 
        return "ArrayHeight"; 
        break;
    case Setting::ArrayWidth : return "ArrayWidth"; break;
    case Setting::MomentumX : return "MomentumX"; break;
    case Setting::MomentumY : return "MomentumY"; break;
    case Setting::PosX : return "PosX"; break;
    case Setting::PosY : return "PosY"; break;
    case Setting::None : return "None"; break;
    }
}

Setting fromStr(std::string s)
{
    if (s == "ArrayWidth") return Setting::ArrayWidth;
    if (s == "ArrayHeight") return Setting::ArrayHeight;
    if (s == "MomentumX") return Setting::MomentumX;
    if (s == "MomentumY") return Setting::MomentumY;
    if (s == "PosX") return Setting::PosX;
    if (s == "PosY") return Setting::PosY;
    return Setting::None;
}

void Config::saveTo(std::string filename)
{

}

void Config::loadFrom(std::string filename)
{
    std::cout << "Loading form config" << std::endl;
    {
        std::ifstream file(filename);
        while (file)
        {
            std::string line;
            std::getline(file, line, '\n');
            std::cout << line << std::endl;
        }
        file.close();
    }

    s_filename = filename;
    std::ifstream file(s_filename);
    while (file)
    {
        std::string key;
        std::getline(file, key, '=');
        std::string value;
        std::getline(file, value, '\n');
        
        Setting s = fromStr(key);
        if (s != Setting::None)
        {
            setValue(s, value);
            std::cout << "Key: " << key << std::endl;
            std::cout << "Value: " << value << std::endl;
        }
        else
        {
            std::cout << "The setting \"" << key << "\" is unknown!" << std::endl;
            break;
        }
    }
}

void Config::setValue(Setting setting, std::string value)
{
    switch (setting)
    {
        case Setting::ArrayWidth:
        {
            unsigned width = std::stoi(value);
            s_arrayWidth = width;
            break;
        }
        
        case Setting::ArrayHeight:
        {
            int height = std::stoi(value);
            s_arrayHeight = height;
            break;
        }
    }
}


int Config::getArrayWidth(){return s_arrayWidth;};
int Config::getArrayHeight(){return s_arrayHeight;};
unsigned Config::getArraySize(){return Config::getArrayHeight() * Config::getArrayWidth();};
int Config::getNumberOfThreads(){return s_numberOfThreads;};
double Config::getHbar(){return s_hBar;};
double Config::getInitialMomentumX(){return s_initalMimentumX;};
double Config::getInitalMomentumY(){return s_initialMomentumY;};
double Config::getInitialPosX(){return s_initialPosX;};
double Config::getInitialPosY(){return s_initialPosY;};
double Config::getInitialLambdaUnsharpness(){return s_initialLambdaUnsharpness;};
double Config::getInitialValueScaleFactor(){return s_initialValueScaleFactor;};
double Config::getBatScale(){return s_batScale;};
double Config::getBatHeight(){return s_batHeight;};
double Config::getBatCutOff(){return s_batCatOff;};
double Config::getTimeStepSize(){return s_timeStepSize;};
double Config::getFlattenMomentumCutOff(){return s_flattenMomentumCutOff;};
double Config::getFlattenMomentumFallOff(){return s_flattenMomentumFallOff;};
double Config::getMomentumImageScale(){return s_momentumImageScale;};
