///
/// @file   Config.hpp
/// @author Armin Co
/// @brief  Load configuration form a file and make it available.
///

#ifndef QPONG_CONFIG_HPP
#define QPONG_CONFIG_HPP


#include<string>
#include<map>
#include<vector>

namespace QPong {


enum class Setting
{
    // Array configuration
    ArrayWidth,
    ArrayHeight,
    // Initial particle settings
    MomentumX,
    MomentumY,
    PosX,
    PosY,
    None
};


const char* toStr(Setting setting);

Setting fromStr(std::string s);

class Config
{
public:
    static void loadFrom(std::string s);


    static void saveTo(std::string s);

    
    static void setValue(Setting setting, std::string value);


    static std::string& getFilename();

    static int getArrayWidth();
    static int getArrayHeight();
    static unsigned getArraySize();
    static int getNumberOfThreads();

    static double getHbar();
    static double getInitialMomentumX();
    static double getInitalMomentumY();
    static double getInitialPosX();
    static double getInitialPosY();
    static double getInitialLambdaUnsharpness();
    static double getInitialValueScaleFactor();

    static double getBatScale();
    static double getBatHeight();
    static double getBatCutOff();
 
    static double getTimeStepSize();
    static double getFlattenMomentumCutOff();
    static double getFlattenMomentumFallOff();
 
    static double getMomentumImageScale();

private:
    inline static std::string s_filename = "qpong.config";

    inline static unsigned s_arrayWidth = 384;
    inline static unsigned s_arrayHeight = 384;
    inline static unsigned s_numberOfThreads = 8;
    inline static double s_hBar = 0.0002;
    inline static double s_initalMimentumX = 0.063;
    inline static double s_initialMomentumY = 0.042;
    inline static double s_initialPosX = 0.0;
    inline static double s_initialPosY = 0.0;
    inline static double s_initialLambdaUnsharpness = 16.0;
    inline static double s_initialValueScaleFactor = 0.5;
    inline static double s_batScale = 0.000015;
    inline static double s_batHeight = 0.31;
    inline static double s_batCatOff = 0.0015;
    inline static double s_timeStepSize = 0.15;
    inline static double s_flattenMomentumCutOff = 0.012;
    inline static double s_flattenMomentumFallOff = 2.0;
    inline static double s_momentumImageScale = 55580.0;

};
}
#endif