///
/// @file   ImageBuffer.cpp
///

#include "Common.hpp"
#include "ImageBuffer.hpp"

using namespace QPong;

ImageBuffer::ImageBuffer()
{
    m_potbuf = new double[QPong::arraySize];
    m_complexBuffer = new std::complex<double>[QPong::arraySize];
}


void ImageBuffer::updateAt(unsigned index, std::complex<double> v, double obs)
{
    m_potbuf[index] = obs;
    m_complexBuffer[index] = v;
}


const QColor ImageBuffer::getValue(unsigned index)
{
    std::complex<double> val = m_complexBuffer[index];
    double r = real(val);
    double i = imag(val);
    double obs = m_potbuf[index];
    
    QColor c {
        static_cast<float>(obs + r*r),          //< red
        static_cast<float>(r*r + i*i + obs/1.5),  //< green
        static_cast<float>(i*i)                 //< blue
    };

    constexpr double max = 0.81;
    if (c.r > max)
    {
        c.r = max;
    }

    if (c.g > max)
    {
        c.g = max;
    }

    if (c.b > max)
    {
        c.b = max;
    }       
    return c;
}
