///
/// @file   ImageBuffer.hpp
/// @author Armin Co
/// @brief  An imagebuffer object stores an image as an array of color (RGB) values.
///

#ifndef QPONG_IMAGE_BUFFER_HPP
#define QPONG_IMAGE_BUFFER_HPP

#include <complex>

namespace QPong
{

/// @brief Convinient conatiner to store a RGB value.
///        {r, g, b}
struct QColor
{
    float r = 0.0f;
    float g = 0.0f;
    float b = 0.0f;
};

/// @brief Class ImageBuffer
///
class ImageBuffer
{
public:
    /// @brief Constructor for ImageBuffer.
    ///
    ImageBuffer();

    /// @brief Update the buffer at the given position.
    /// @param index Index of the value that should be changed.
    /// @param v The value as a complex number which will be interpreted as a color.
    /// @param pot Potential at position.
    ///
    void updateAt(unsigned index, std::complex<double> v, double pot);

    /// @return Get color value from particle at array index.
    ///
    const QColor getValue(unsigned index);

private:
    double *m_potbuf;
    std::complex<double> *m_complexBuffer;
};

} // namespace QPong

#endif
