///
/// @file       QPlayer.cpp
/// @author     Armin Co
///


#include <mutex>
#include "Player.hpp"

using namespace QPong;

constexpr double speedStepSize = 0.0005;


Player::Player(Player::Id pl, Player::Position pos)
    : m_id {pl}
    , m_x {pos.x}
    , m_y {pos.y}
{
}


void Player::update(double dt)
{
    constexpr double negativeAccelScale = 1.5;
    m_y += (m_vy * dt * 4);
    
    if (m_vy > 0.0)
    {
        m_vy -= speedStepSize * negativeAccelScale;
    }
    if (m_vy < 0.0)
    {
        m_vy += speedStepSize * negativeAccelScale;
    }
    if (m_vy < speedStepSize*negativeAccelScale && m_vy > -speedStepSize * negativeAccelScale)
    {
        m_vy = 0.0;
    }

    constexpr double maxY = 1.3;
    if (m_y > maxY)
    {
        m_y = maxY;
    }
    if (m_y < -maxY)
    {
        m_y = -maxY;
    }
}


void Player::move(Direction direction)
{
    constexpr double addSpped = 3;
    switch (direction)
    {
    case Direction::Up:
        m_vy -= speedStepSize * addSpped;
        break;
    case Direction::Down:
        m_vy += speedStepSize * addSpped;
        break;
    }
    
    constexpr double maxSpeedScale = 45;
    if (m_vy > speedStepSize * maxSpeedScale)
    {
        m_vy = speedStepSize * maxSpeedScale;
    }
    if (m_vy < -speedStepSize * maxSpeedScale)
    {
        m_vy = -speedStepSize * maxSpeedScale;
    }
}


float Player::getScore()
{
    return m_score * 100;
}


std::mutex scoreMtx;
void Player::addScore(float score)
{
    std::lock_guard<std::mutex> lockScore(scoreMtx);
    m_score += score;
}


Player::Position Player::getPosition()
{
    return {m_x, m_y};
}


void Player::reset()
{
    m_score = 0;
}